Package: tomoseqr
Title: R Package for Analyzing Tomo-seq Data
Version: 1.15.0
Authors@R: 
    person(
        given = "Ryosuke",
        family = "Matsuzawa",
        role = c("aut", "cre"),
        email = "kinakomochi.work@gmail.com",
        comment = c(ORCID = "0000-0003-2999-4734")
    )
Description: `tomoseqr` is an R package for analyzing Tomo-seq data.
    Tomo-seq is a genome-wide RNA tomography method that combines combining
    high-throughput RNA sequencing with cryosectioning for spatially resolved transcriptomics.
    `tomoseqr` reconstructs 3D expression patterns from tomo-seq data
    and visualizes the reconstructed 3D expression patterns.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: FALSE
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.2
VignetteBuilder: knitr
Depends: R (>= 4.2)
biocViews: GeneExpression, Sequencing, RNASeq, Transcriptomics,
        Spatial, Visualization, Software
Imports: grDevices, graphics, animation, tibble, dplyr, stringr, purrr,
        methods, shiny, BiocFileCache, readr, tools, plotly, ggplot2
Suggests: rmarkdown, knitr, BiocStyle, testthat (>= 3.0.0)
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/tomoseqr
git_branch: devel
git_last_commit: 5f3f9ac
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:27:12 UTC; biocbuild
Author: Ryosuke Matsuzawa [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2999-4734>)
Maintainer: Ryosuke Matsuzawa <kinakomochi.work@gmail.com>
