\name{mask}
\alias{mask}
\docType{data}
\title{A matrix containing mask data.}
\description{
    It is a matrix representing the mask. Each element of the matrix
    corresponds to a voxel, and has the value 1 if the voxel is included
    in the mask, 0 if it is not.
}