% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/masker.R
\name{masker}
\alias{masker}
\title{Make mask}
\usage{
masker()
}
\value{
NA
}
\description{
Make mask
}
\examples{
if (interactive()) {
    masker()
}
}
