\name{tomoObj}
\alias{tomoObj}
\docType{data}
\title{A tomoSeq object.}
\description{
    It is a tomoSeq object generated from \code{\link[tomoseqr]{testx}},
    \code{\link[tomoseqr]{testy}}, \code{\link[tomoseqr]{testz}} and
    \code{\link[tomoseqr]{mask}}.
}