% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_function.R
\name{toDataFrame}
\alias{toDataFrame}
\title{Convert reconstructed matrix to data.frame.}
\usage{
toDataFrame(tomoObj, geneID)
}
\arguments{
\item{tomoObj}{tomoSeq object}

\item{geneID}{single gene ID}
}
\value{
Reconstruction result converted to dataframe.
}
\description{
Convert reconstructed matrix to data.frame.
}
\examples{
data(tomoObj)
toDataFrame(tomoObj, "gene2")
}
